@@ -34,14 +34,13 @@
 
 
 #include "php_yaml.h"
-#include "zval_refcount.h" /* for PHP < 5.3 */
 #include "php_yaml_int.h"
 
 /* {{{ local macros
  */
 #define y_event_init_failed(e) \
   yaml_event_delete(e); \
-  php_error_docref(NULL TSRMLS_CC, E_WARNING,\
+  php_error_docref(NULL, E_WARNING,\
 	  "Memory error: Not enough memory for creating an event (libyaml)")
 
 #define Y_ARRAY_SEQUENCE 1
@@ -52,34 +51,34 @@
 /* {{{ local prototypes
  */
 static int y_event_emit(
-		const y_emit_state_t *state, yaml_event_t *event TSRMLS_DC);
-static void y_handle_emitter_error(const y_emit_state_t *state TSRMLS_DC);
-static int y_array_is_sequence(HashTable *ht TSRMLS_DC);
-static void y_scan_recursion(const y_emit_state_t *state, zval *data TSRMLS_DC);
-static long y_search_recursive(
-		const y_emit_state_t *state, const unsigned long addr TSRMLS_DC);
+		const y_emit_state_t *state, yaml_event_t *event);
+static void y_handle_emitter_error(const y_emit_state_t *state);
+static int y_array_is_sequence(HashTable *ht);
+static void y_scan_recursion(const y_emit_state_t *state, zval *data);
+static zend_long y_search_recursive(
+		const y_emit_state_t *state, const zend_ulong addr);
 
 static int y_write_zval(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_null(
-		const y_emit_state_t *state, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, yaml_char_t *tag);
 static int y_write_bool(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_long(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_double(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_string(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_array(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_timestamp(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_object(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC);
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag);
 static int y_write_object_callback (
 		const y_emit_state_t *state, zval *callback, zval *data,
-		const char *clazz_name TSRMLS_DC);
+		const char *clazz_name);
 static inline unsigned int get_next_char(
 		const unsigned char *str, size_t str_len, size_t *cursor, int *status);
 /* }}} */
@@ -89,11 +88,11 @@ static inline unsigned int get_next_char(
  * send an event to the emitter
  */
 static int
-y_event_emit(const y_emit_state_t *state, yaml_event_t *event TSRMLS_DC)
+y_event_emit(const y_emit_state_t *state, yaml_event_t *event)
 {
 	if (!yaml_emitter_emit(state->emitter, event)) {
 		yaml_event_delete(event);
-		y_handle_emitter_error(state TSRMLS_CC);
+		y_handle_emitter_error(state);
 		return FAILURE;
 
 	} else {
@@ -106,26 +105,26 @@ y_event_emit(const y_emit_state_t *state, yaml_event_t *event TSRMLS_DC)
 /* {{{ y_handle_emitter_error()
  * Emit a warning about an emitter error
  */
-static void y_handle_emitter_error(const y_emit_state_t *state TSRMLS_DC)
+static void y_handle_emitter_error(const y_emit_state_t *state)
 {
 	switch (state->emitter->error) {
 	case YAML_MEMORY_ERROR:
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,
+		php_error_docref(NULL, E_WARNING,
 				"Memory error: Not enough memory for emitting");
 		break;
 
 	case YAML_WRITER_ERROR:
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,
+		php_error_docref(NULL, E_WARNING,
 				"Writer error: %s", state->emitter->problem);
 		break;
 
 	case YAML_EMITTER_ERROR:
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,
+		php_error_docref(NULL, E_WARNING,
 				"Emitter error: %s", state->emitter->problem);
 		break;
 
 	default:
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Internal error");
+		php_error_docref(NULL, E_WARNING, "Internal error");
 		break;
 	}
 }
@@ -135,30 +134,23 @@ static void y_handle_emitter_error(const y_emit_state_t *state TSRMLS_DC)
 /* {{{ y_array_is_sequence()
  * Does the array encode a sequence?
  */
-static int y_array_is_sequence(HashTable *ht TSRMLS_DC)
+static int y_array_is_sequence(HashTable *ht)
 {
-	HashPosition pos;
-	ulong kidx, idx;
-	const char *kstr;
-	int key_type;
+	zend_ulong kidx, idx;
+	zend_string *str_key;
 
-	zend_hash_internal_pointer_reset_ex(ht, &pos);
 	idx = 0;
-	while (SUCCESS == zend_hash_has_more_elements_ex(ht, &pos)) {
-		key_type = zend_hash_get_current_key_ex(
-				ht, (char **) &kstr, NULL, &kidx, 0, &pos);
-		if (HASH_KEY_IS_LONG != key_type) {
+	ZEND_HASH_FOREACH_KEY(ht, kidx, str_key) {
+		if (str_key) {
 			/* non-numeric key found */
 			return Y_ARRAY_MAP;
-
 		} else if (kidx != idx) {
 			/* gap in sequence found */
 			return Y_ARRAY_MAP;
 		}
 
 		idx++;
-		zend_hash_move_forward_ex(ht, &pos);
-	};
+	} ZEND_HASH_FOREACH_END();
 	return Y_ARRAY_SEQUENCE;
 }
 /* }}} */
@@ -167,11 +159,12 @@ static int y_array_is_sequence(HashTable *ht TSRMLS_DC)
 /* {{{ y_scan_recursion()
  * walk an object graph looking for recursive references
  */
-static void y_scan_recursion(const y_emit_state_t *state, zval *data TSRMLS_DC)
+static void y_scan_recursion(const y_emit_state_t *state, zval *data)
 {
 	HashTable *ht;
-	HashPosition pos;
-	zval **elm;
+	zval *elm;
+
+	ZVAL_DEREF(data);
 
 	ht = HASH_OF(data);
 
@@ -180,25 +173,42 @@ static void y_scan_recursion(const y_emit_state_t *state, zval *data TSRMLS_DC)
 		return;
 	}
 
-	if (ht->nApplyCount > 0) {
-		zval *tmp;
-		MAKE_STD_ZVAL(tmp);
-		ZVAL_LONG(tmp, (unsigned long) ht);
+#if PHP_VERSION_ID >= 70300
+	if (!(GC_FLAGS(ht) & GC_IMMUTABLE) && GC_IS_RECURSIVE(ht)) {
+#else
+	if (ZEND_HASH_APPLY_PROTECTION(ht) && ht->u.v.nApplyCount > 0) {
+#endif
+		zval tmp;
+		ZVAL_LONG(&tmp, (zend_ulong) ht);
 
 		/* we've seen this before, so record address */
-		zend_hash_next_index_insert(
-				state->recursive, &tmp, sizeof(zval *), NULL);
+		zend_hash_next_index_insert(state->recursive, &tmp);
 		return;
 	}
 
-	ht->nApplyCount++;
-	zend_hash_internal_pointer_reset_ex(ht, &pos);
-	while (SUCCESS == zend_hash_has_more_elements_ex(ht, &pos)) {
-		zend_hash_get_current_data_ex(ht, (void **) &elm, &pos);
-		y_scan_recursion(state, (*elm) TSRMLS_CC);
-		zend_hash_move_forward_ex(ht, &pos);
-	};
-	ht->nApplyCount--;
+#if PHP_VERSION_ID >= 70300
+	if (!(GC_FLAGS(ht) & GC_IMMUTABLE)) {
+		GC_PROTECT_RECURSION(ht);
+	}
+#else
+	if (ZEND_HASH_APPLY_PROTECTION(ht)) {
+		ht->u.v.nApplyCount++;
+	}
+#endif
+
+	ZEND_HASH_FOREACH_VAL(ht, elm) {
+		y_scan_recursion(state, elm);
+	} ZEND_HASH_FOREACH_END();
+
+#if PHP_VERSION_ID >= 70300
+	if (!(GC_FLAGS(ht) & GC_IMMUTABLE)) {
+		GC_UNPROTECT_RECURSION(ht);
+	}
+#else
+	if (ZEND_HASH_APPLY_PROTECTION(ht)) {
+		ht->u.v.nApplyCount--;
+	}
+#endif
 
 	return;
 }
@@ -208,25 +218,19 @@ static void y_scan_recursion(const y_emit_state_t *state, zval *data TSRMLS_DC)
 /* {{{ y_search_recursive()
  * Search the recursive state hash for an address
  */
-static long y_search_recursive(
-		const y_emit_state_t *state, const unsigned long addr TSRMLS_DC)
+static zend_long y_search_recursive(
+		const y_emit_state_t *state, const zend_ulong addr)
 {
- 	zval **entry;
-	HashPosition pos;
-   	ulong num_key;
-	unsigned long found;
+	zval *entry;
+	zend_ulong num_key;
+	zend_ulong found;
 
-	zend_hash_internal_pointer_reset_ex(state->recursive, &pos);
-	while (SUCCESS == zend_hash_has_more_elements_ex(state->recursive, &pos)) {
-		zend_hash_get_current_data_ex(state->recursive, (void **)&entry, &pos);
-		found = (unsigned long) Z_LVAL_PP(entry);
+	ZEND_HASH_FOREACH_NUM_KEY_VAL(state->recursive, num_key, entry) {
+		found = Z_LVAL_P(entry);
 		if (addr == found) {
-			zend_hash_get_current_key_ex(
-					state->recursive, NULL, NULL, &num_key, 0, &pos);
 			return num_key;
 		}
-		zend_hash_move_forward_ex(state->recursive, &pos);
-	}
+	} ZEND_HASH_FOREACH_END();
 	return -1;
 }
 /* }}} */
@@ -236,46 +240,51 @@ static long y_search_recursive(
  * Write a php zval to the emitter
  */
 static int y_write_zval(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	int status = FAILURE;
 
 	switch (Z_TYPE_P(data)) {
-	case IS_NULL:
-		status = y_write_null(state, tag TSRMLS_CC);
+	case IS_REFERENCE:
+		status = y_write_zval(state, Z_REFVAL_P(data), tag);
 		break;
 
-	case IS_BOOL:
-		status = y_write_bool(state, data, tag TSRMLS_CC);
+	case IS_NULL:
+		status = y_write_null(state, tag);
+		break;
+
+	case IS_TRUE:
+	case IS_FALSE:
+		status = y_write_bool(state, data, tag);
 		break;
 
 	case IS_LONG:
-		status = y_write_long(state, data, tag TSRMLS_CC);
+		status = y_write_long(state, data, tag);
 		break;
 
 	case IS_DOUBLE:
-		status = y_write_double(state, data, tag TSRMLS_CC);
+		status = y_write_double(state, data, tag);
 		break;
 
 	case IS_STRING:
-		status = y_write_string(state, data, tag TSRMLS_CC);
+		status = y_write_string(state, data, tag);
 		break;
 
 	case IS_ARRAY:
-		status = y_write_array(state, data, tag TSRMLS_CC);
+		status = y_write_array(state, data, tag);
 		break;
 
 	case IS_OBJECT:
-		status = y_write_object(state, data, tag TSRMLS_CC);
+		status = y_write_object(state, data, tag);
 		break;
 
 	case IS_RESOURCE:		/* unsupported object */
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE,
+		php_error_docref(NULL, E_NOTICE,
 				"Unable to emit PHP resources.");
 		break;
 
 	default:				/* something we didn't think of */
-		php_error_docref(NULL TSRMLS_CC, E_NOTICE,
+		php_error_docref(NULL, E_NOTICE,
 				"Unsupported php zval type %d.", Z_TYPE_P(data));
 		break;
 	}
@@ -287,7 +296,7 @@ static int y_write_zval(
 
 /* {{{ y_write_null()
  */
-static int y_write_null(const y_emit_state_t *state, yaml_char_t *tag TSRMLS_DC)
+static int y_write_null(const y_emit_state_t *state, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int omit_tag = 0;
@@ -305,7 +314,7 @@ static int y_write_null(const y_emit_state_t *state, yaml_char_t *tag TSRMLS_DC)
 		y_event_init_failed(&event);
 		return FAILURE;
 	}
-	return y_event_emit(state, &event TSRMLS_CC);
+	return y_event_emit(state, &event);
 }
 /* }}} */
 
@@ -313,12 +322,12 @@ static int y_write_null(const y_emit_state_t *state, yaml_char_t *tag TSRMLS_DC)
 /* {{{ y_write_bool()
  */
 static int y_write_bool(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int omit_tag = 0;
 	int status;
-	const char *res = Z_BVAL_P(data) ? "true" : "false";
+	const char *res = Z_TYPE_P(data) == IS_TRUE ? "true" : "false";
 
 	if (NULL == tag) {
 		tag = (yaml_char_t *) YAML_BOOL_TAG;
@@ -332,7 +341,7 @@ static int y_write_bool(
 		y_event_init_failed(&event);
 		return FAILURE;
 	}
-	return y_event_emit(state, &event TSRMLS_CC);
+	return y_event_emit(state, &event);
 }
 /* }}} */
 
@@ -340,7 +349,7 @@ static int y_write_bool(
 /* {{{ y_write_long()
  */
 static int y_write_long(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int omit_tag = 0;
@@ -353,9 +362,9 @@ static int y_write_long(
 		omit_tag = 1;
 	}
 
-	res_size = snprintf(res, 0, "%ld", Z_LVAL_P(data));
+	res_size = snprintf(res, 0, ZEND_LONG_FMT, Z_LVAL_P(data));
 	res = (char*) emalloc(res_size + 1);
-	snprintf(res, res_size + 1, "%ld", Z_LVAL_P(data));
+	snprintf(res, res_size + 1, ZEND_LONG_FMT, Z_LVAL_P(data));
 
 	status = yaml_scalar_event_initialize(&event, NULL, tag,
 			(yaml_char_t *) res, strlen(res),
@@ -365,7 +374,7 @@ static int y_write_long(
 		y_event_init_failed(&event);
 		return FAILURE;
 	}
-	return y_event_emit(state, &event TSRMLS_CC);
+	return y_event_emit(state, &event);
 }
 /* }}} */
 
@@ -373,32 +382,30 @@ static int y_write_long(
 /* {{{ y_write_double()
  */
 static int y_write_double(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int omit_tag = 0;
 	int status;
-	char *res = { 0 };
-	size_t res_size;
+	char res[PHP_DOUBLE_MAX_LENGTH];
 
 	if (NULL == tag) {
 		tag = (yaml_char_t *) YAML_FLOAT_TAG;
 		omit_tag = 1;
 	}
 
-	res_size = snprintf(res, 0, "%f", Z_DVAL_P(data));
-	res = (char*) emalloc(res_size + 1);
-	snprintf(res, res_size + 1, "%f", Z_DVAL_P(data));
+	// Bug 79866: let PHP determine output precision
+	php_gcvt(Z_DVAL_P(data), (int)PG(serialize_precision), '.', 'E', res);
 
 	status = yaml_scalar_event_initialize(&event, NULL, tag,
 			(yaml_char_t *) res, strlen(res),
 			omit_tag, omit_tag, YAML_PLAIN_SCALAR_STYLE);
-	efree(res);
+
 	if (!status) {
 		y_event_init_failed(&event);
 		return FAILURE;
 	}
-	return y_event_emit(state, &event TSRMLS_CC);
+	return y_event_emit(state, &event);
 }
 /* }}} */
 
@@ -406,7 +413,7 @@ static int y_write_double(
 /* {{{ y_write_string()
  */
 static int y_write_string(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int omit_tag = 0;
@@ -432,7 +439,7 @@ static int y_write_string(
 			us = get_next_char(s, len, &pos, &status);
 			if (status != SUCCESS) {
 				/* invalid UTF-8 character found */
-				php_error_docref(NULL TSRMLS_CC, E_WARNING,
+				php_error_docref(NULL, E_WARNING,
 						"Invalid UTF-8 sequence in argument");
 				return FAILURE;
 
@@ -450,7 +457,7 @@ static int y_write_string(
 		y_event_init_failed(&event);
 		return FAILURE;
 	}
-	return y_event_emit(state, &event TSRMLS_CC);
+	return y_event_emit(state, &event);
 }
 /* }}} */
 
@@ -458,25 +465,22 @@ static int y_write_string(
 /* {{{ y_write_array()
  */
 static int y_write_array(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int omit_tag = 0;
 	int status;
 	HashTable *ht = Z_ARRVAL_P(data);
-	HashPosition pos;
-	zval **elm;
+	zval *elm;
 	int array_type;
 	zval key_zval;
-	ulong kidx;
-	uint key_len;
-	char *kstr = { 0 };
-	HashTable *tmp_ht;
-	long recursive_idx = -1;
+	zend_ulong kidx;
+	zend_string *kstr;
+	zend_long recursive_idx = -1;
 	char *anchor = { 0 };
 	size_t anchor_size;
 
-	array_type = y_array_is_sequence(ht TSRMLS_CC);
+	array_type = y_array_is_sequence(ht);
 
 	if (NULL == tag) {
 		if (Y_ARRAY_SEQUENCE == array_type) {
@@ -497,14 +501,18 @@ static int y_write_array(
 	 *   if ht->nApplyCount > 0:
 	 *     emit a ref
 	 */
-	recursive_idx = y_search_recursive(state, (unsigned long) ht TSRMLS_CC);
+	recursive_idx = y_search_recursive(state, (zend_ulong) ht);
 	if (-1 != recursive_idx) {
 		/* create anchor to refer to this structure */
-		anchor_size = snprintf(anchor, 0, "refid%ld", recursive_idx + 1);
+		anchor_size = snprintf(anchor, 0, "refid" ZEND_LONG_FMT, recursive_idx + 1);
 		anchor = (char*) emalloc(anchor_size + 1);
-		snprintf(anchor, anchor_size + 1, "refid%ld", recursive_idx + 1);
+		snprintf(anchor, anchor_size + 1, "refid" ZEND_LONG_FMT, recursive_idx + 1);
 
-		if (ht->nApplyCount > 1) {
+#if PHP_VERSION_ID >= 70300
+		if (!(GC_FLAGS(ht) & GC_IMMUTABLE) && GC_IS_RECURSIVE(ht)) {
+#else
+		if (ZEND_HASH_APPLY_PROTECTION(ht) && ht->u.v.nApplyCount > 0) {
+#endif
 			/* node has been visited before */
 			status = yaml_alias_event_initialize(
 					&event, (yaml_char_t *) anchor);
@@ -514,7 +522,7 @@ static int y_write_array(
 				return FAILURE;
 			}
 
-			status = y_event_emit(state, &event TSRMLS_CC);
+			status = y_event_emit(state, &event);
 			efree(anchor);
 			return status;
 		}
@@ -537,7 +545,7 @@ static int y_write_array(
 		}
 		return FAILURE;
 	}
-	status = y_event_emit(state, &event TSRMLS_CC);
+	status = y_event_emit(state, &event);
 	if (anchor) {
 		efree(anchor);
 	}
@@ -545,51 +553,54 @@ static int y_write_array(
 		return FAILURE;
 	}
 
+#if PHP_VERSION_ID >= 70300
+	if (!(GC_FLAGS(ht) & GC_IMMUTABLE)) {
+		/* increment access count for hash */
+		GC_PROTECT_RECURSION(ht);
+	}
+#else
+	if (ZEND_HASH_APPLY_PROTECTION(ht)) {
+		/* increment access count for hash */
+		ht->u.v.nApplyCount++;
+	}
+#endif
+
 	/* emit array elements */
-	zend_hash_internal_pointer_reset_ex(ht, &pos);
-	while (SUCCESS == zend_hash_has_more_elements_ex(ht, &pos)) {
+	ZEND_HASH_FOREACH_KEY_VAL(ht, kidx, kstr, elm) {
+		ZVAL_DEREF(elm);
+
 		if (Y_ARRAY_MAP == array_type) {
-			zend_hash_get_current_key_ex(ht, (char **) &kstr, &key_len, &kidx,
-					0, &pos);
-
 			/* create zval for key */
-			if (HASH_KEY_IS_LONG ==
-					zend_hash_get_current_key_type_ex(ht, &pos)) {
-				ZVAL_LONG(&key_zval, kidx);
-
+			if (kstr) {
+				ZVAL_STR(&key_zval, kstr);
 			} else {
-				ZVAL_STRINGL(&key_zval, kstr, strlen(kstr), 1);
+				ZVAL_LONG(&key_zval, kidx);
 			}
 
 			/* emit key */
-			status = y_write_zval(state, &key_zval, NULL TSRMLS_CC);
-			zval_dtor(&key_zval);
+			status = y_write_zval(state, &key_zval, NULL);
 			if (SUCCESS != status) {
 				return FAILURE;
 			}
 		}
 
-		if (SUCCESS ==
-				zend_hash_get_current_data_ex(ht, (void **) &elm, &pos)) {
+		status = y_write_zval(state, elm, NULL);
 
-			tmp_ht = HASH_OF(*elm);
-			if (tmp_ht) {
-				/* increment access count for hash */
-				tmp_ht->nApplyCount++;
-			}
 
-			status = y_write_zval(state, (*elm), NULL TSRMLS_CC);
-
-			if (tmp_ht) {
-				tmp_ht->nApplyCount--;
-			}
-
-			if (SUCCESS != status) {
-				return FAILURE;
-			}
+		if (SUCCESS != status) {
+			return FAILURE;
 		}
-		zend_hash_move_forward_ex(ht, &pos);
-	};
+	} ZEND_HASH_FOREACH_END();
+
+#if PHP_VERSION_ID >= 70300
+	if (!(GC_FLAGS(ht) & GC_IMMUTABLE)) {
+		GC_UNPROTECT_RECURSION(ht);
+	}
+#else
+	if (ZEND_HASH_APPLY_PROTECTION(ht)) {
+		ht->u.v.nApplyCount--;
+	}
+#endif
 
 	if (Y_ARRAY_SEQUENCE == array_type) {
 		status = yaml_sequence_end_event_initialize(&event);
@@ -601,7 +612,7 @@ static int y_write_array(
 		y_event_init_failed(&event);
 		return FAILURE;
 	}
-	return y_event_emit(state, &event TSRMLS_CC);
+	return y_event_emit(state, &event);
 }
 /* }}} */
 
@@ -609,13 +620,13 @@ static int y_write_array(
 /* y_write_timestamp()
  */
 static int y_write_timestamp(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int omit_tag = 0;
 	int status;
 	zend_class_entry *clazz = Z_OBJCE_P(data);
-	zval *timestamp = { 0 };
+	zval timestamp = {{0} };
 	zval dtfmt;
 
 	if (NULL == tag) {
@@ -623,28 +634,27 @@ static int y_write_timestamp(
 		omit_tag = 1;
 	}
 
-	/* get iso-8601 format specifier */
-#if ZEND_MODULE_API_NO >= 20071006
-	zend_get_constant_ex("DateTime::ISO8601", 17, &dtfmt, clazz, 0 TSRMLS_CC);
-#else
-	zend_get_constant_ex("DateTime::ISO8601", 17, &dtfmt, clazz TSRMLS_CC);
-#endif
+	/* iso-8601 format specifier including milliseconds */
+	ZVAL_STRING(&dtfmt, "Y-m-d\\TH:i:s.uP");
+
 	/* format date as iso-8601 string */
-	zend_call_method_with_1_params(&data, clazz, NULL,
-			"format", &timestamp, &dtfmt);
-	zval_dtor(&dtfmt);
+#if PHP_VERSION_ID >= 80000
+	zend_call_method_with_1_params(Z_OBJ_P(data), clazz, NULL, "format", &timestamp, &dtfmt);
+#else
+	zend_call_method_with_1_params(data, clazz, NULL, "format", &timestamp, &dtfmt);
+#endif
+	zval_ptr_dtor(&dtfmt);
 
 	/* emit formatted date */
 	status = yaml_scalar_event_initialize(&event, NULL, tag,
-			(yaml_char_t *) Z_STRVAL_P(timestamp), Z_STRLEN_P(timestamp),
+			(yaml_char_t *) Z_STRVAL_P(&timestamp), Z_STRLEN_P(&timestamp),
 			omit_tag, omit_tag, YAML_PLAIN_SCALAR_STYLE);
-	zval_dtor(timestamp);
-	efree(timestamp);
+	zval_ptr_dtor(&timestamp);
 	if (!status) {
 		y_event_init_failed(&event);
 		return FAILURE;
 	}
-	return y_event_emit(state, &event TSRMLS_CC);
+	return y_event_emit(state, &event);
 }
 /* }}} */
 
@@ -652,47 +662,44 @@ static int y_write_timestamp(
 /* {{{ y_write_object()
  */
 static int y_write_object(
-		const y_emit_state_t *state, zval *data, yaml_char_t *tag TSRMLS_DC)
+		const y_emit_state_t *state, zval *data, yaml_char_t *tag)
 {
 	yaml_event_t event;
 	int status;
-	const char *clazz_name = { 0 };
-	zend_uint name_len;
-	zval **callback = { 0 };
+	zend_string *clazz_name;
+	zval *callback;
 
-	zend_get_object_classname(data, &clazz_name, &name_len TSRMLS_CC);
+	clazz_name = Z_OBJCE_P(data)->name;
 
 	/* TODO check for a "yamlSerialize()" instance method */
-	if (NULL != state->callbacks && SUCCESS == zend_hash_find(
-			state->callbacks, clazz_name, name_len + 1, (void **) &callback)) {
+	if (NULL != state->callbacks && (callback = zend_hash_find(
+			state->callbacks, clazz_name)) != NULL) {
 		/* found a registered callback for this class */
 		status = y_write_object_callback(
-				state, *callback, data, clazz_name TSRMLS_CC);
-
-	} else if (0 == strncmp(clazz_name, "DateTime", name_len)) {
-		status = y_write_timestamp(state, data, tag TSRMLS_CC);
+				state, callback, data, clazz_name->val);
 
+	} else if (0 == strncmp(clazz_name->val, "DateTime", clazz_name->len)) {
+		status = y_write_timestamp(state, data, tag);
 	} else {
 		/* tag and emit serialized version of object */
 		php_serialize_data_t var_hash;
 		smart_str buf = { 0 };
 
 		PHP_VAR_SERIALIZE_INIT(var_hash);
-		php_var_serialize(&buf, &data, &var_hash TSRMLS_CC);
+		php_var_serialize(&buf, data, &var_hash);
 		PHP_VAR_SERIALIZE_DESTROY(var_hash);
 
 		status = yaml_scalar_event_initialize(&event,
-				NULL, (yaml_char_t *) YAML_PHP_TAG,
-				(yaml_char_t *) buf.c, buf.len,
+				NULL, (yaml_char_t *) YAML_PHP_TAG, (yaml_char_t *) buf.s->val, buf.s->len,
 				0, 0, YAML_DOUBLE_QUOTED_SCALAR_STYLE);
 
-		smart_str_free(&buf);
 		if (!status) {
 			y_event_init_failed(&event);
 			status = FAILURE;
 		} else {
-			status = y_event_emit(state, &event TSRMLS_CC);
+			status = y_event_emit(state, &event);
 		}
+		smart_str_free(&buf);
 	}
 
 	return status;
@@ -704,53 +711,59 @@ static int y_write_object(
 static int
 y_write_object_callback (
 		const y_emit_state_t *state, zval *callback, zval *data,
-		const char *clazz_name TSRMLS_DC) {
-	zval **argv[] = { &data };
-	zval *zret = { 0 };
-	zval **ztag = { 0 };
-	zval **zdata = { 0 };
+		const char *clazz_name) {
+	zval argv[1];
+	argv[0] = *data;
+	zval zret;
+	zval *ztag;
+	zval *zdata;
+	zend_string *str_key;
 
 	/* call the user function */
-	if (FAILURE == call_user_function_ex(EG(function_table), NULL,
-			callback, &zret, 1, argv, 0, NULL TSRMLS_CC) ||
-			NULL == zret) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,
+	if (FAILURE == call_user_function(EG(function_table), NULL,
+			callback, &zret, 1, argv) ||
+			Z_TYPE_P(&zret) == IS_UNDEF) {
+		php_error_docref(NULL, E_WARNING,
 				"Failed to apply callback for class '%s'"
 				" with user defined function", clazz_name);
 		return FAILURE;
 	}
 
 	/* return val should be array */
-	if (IS_ARRAY != Z_TYPE_P(zret)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,
+	if (IS_ARRAY != Z_TYPE_P(&zret)) {
+		php_error_docref(NULL, E_WARNING,
 				"Expected callback for class '%s'"
 				" to return an array", clazz_name);
 		return FAILURE;
 	}
 
 	/* pull out the tag and surrogate object */
-	if (SUCCESS != zend_hash_find(
-			Z_ARRVAL_P(zret), "tag", sizeof("tag"), (void **)&ztag) ||
-			IS_STRING != Z_TYPE_PP(ztag)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,
+	str_key = zend_string_init("tag", sizeof("tag") - 1, 0);
+	if ((ztag = zend_hash_find(Z_ARRVAL_P(&zret), str_key)) == NULL ||  Z_TYPE_P(ztag) != IS_STRING) {
+		php_error_docref(NULL, E_WARNING,
 				"Expected callback result for class '%s'"
 				" to contain a key named 'tag' with a string value",
 				clazz_name);
+		zend_string_release(str_key);
 		return FAILURE;
 	}
+	zend_string_release(str_key);
 
-	if (SUCCESS != zend_hash_find(
-			Z_ARRVAL_P(zret), "data", sizeof("data"), (void **)&zdata)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING,
+	str_key = zend_string_init("data", sizeof("data") - 1, 0);
+	if ((zdata = zend_hash_find(Z_ARRVAL_P(&zret), str_key)) == NULL) {
+		php_error_docref(NULL, E_WARNING,
 				"Expected callback result for class '%s'"
 				" to contain a key named 'data'",
 				clazz_name);
+		zend_string_release(str_key);
 		return FAILURE;
 	}
+	zend_string_release(str_key);
+
 
 	/* emit surrogate object and tag */
 	return y_write_zval(
-			state, (*zdata), (yaml_char_t *) Z_STRVAL_PP(ztag) TSRMLS_CC);
+			state, zdata, (yaml_char_t *) Z_STRVAL_P(ztag));
 }
 /* }}} */
 
@@ -760,7 +773,7 @@ y_write_object_callback (
 int
 php_yaml_write_impl(
 		yaml_emitter_t *emitter, zval *data,
-		yaml_encoding_t encoding, HashTable *callbacks TSRMLS_DC)
+		yaml_encoding_t encoding, HashTable *callbacks)
 {
 	y_emit_state_t state;
 	yaml_event_t event;
@@ -770,7 +783,7 @@ php_yaml_write_impl(
 	/* scan for recursive objects */
 	ALLOC_HASHTABLE(state.recursive);
 	zend_hash_init(state.recursive, 8, NULL, NULL, 0);
-	y_scan_recursion(&state, data TSRMLS_CC);
+	y_scan_recursion(&state, data);
 	state.callbacks = callbacks;
 
 
@@ -781,7 +794,7 @@ php_yaml_write_impl(
 		status = FAILURE;
 		goto cleanup;
 	}
-	if (FAILURE == y_event_emit(&state, &event TSRMLS_CC)) {
+	if (FAILURE == y_event_emit(&state, &event)) {
 		status = FAILURE;
 		goto cleanup;
 	}
@@ -793,13 +806,13 @@ php_yaml_write_impl(
 		status = FAILURE;
 		goto cleanup;
 	}
-	if (FAILURE == y_event_emit(&state, &event TSRMLS_CC)) {
+	if (FAILURE == y_event_emit(&state, &event)) {
 		status = FAILURE;
 		goto cleanup;
 	}
 
 	/* output data */
-	if (FAILURE == y_write_zval(&state, data, NULL TSRMLS_CC)) {
+	if (FAILURE == y_write_zval(&state, data, NULL)) {
 		status = FAILURE;
 		goto cleanup;
 	}
@@ -811,7 +824,7 @@ php_yaml_write_impl(
 		status = FAILURE;
 		goto cleanup;
 	}
-	if (FAILURE == y_event_emit(&state, &event TSRMLS_CC)) {
+	if (FAILURE == y_event_emit(&state, &event)) {
 		status = FAILURE;
 		goto cleanup;
 	}
@@ -823,7 +836,7 @@ php_yaml_write_impl(
 		status = FAILURE;
 		goto cleanup;
 	}
-	if (FAILURE == y_event_emit(&state, &event TSRMLS_CC)) {
+	if (FAILURE == y_event_emit(&state, &event)) {
 		status = FAILURE;
 		goto cleanup;
 	}
@@ -846,7 +859,7 @@ cleanup:
 int
 php_yaml_write_to_buffer(void *data, unsigned char *buffer, size_t size)
 {
-	smart_str_appendl((smart_str *) data, (char *) buffer, size);
+	smart_string_appendl((smart_string *) data, (char *) buffer, size);
 	return 1;
 }
 /* }}} */
