@@ -53,10 +53,13 @@ extern "C" {
 #include <ext/standard/base64.h>
 #include <ext/standard/basic_functions.h>
 #include <ext/standard/php_var.h>
-#include <ext/standard/php_smart_str.h>
+#include <ext/standard/php_smart_string.h>
 #include <Zend/zend_extensions.h>
 #include <Zend/zend_hash.h>
 #include <Zend/zend_interfaces.h>
+#include <Zend/zend_long.h>
+#include <Zend/zend_smart_str.h>
+#include <Zend/zend_strtod.h>
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
@@ -67,7 +70,7 @@ extern "C" {
 extern "C" {
 #endif
 
-#define PHP_YAML_VERSION "1.2.0"
+#define PHP_YAML_VERSION "2.2.0"
 
 extern zend_module_entry yaml_module_entry;
 #define phpext_yaml_ptr &yaml_module_entry
@@ -76,15 +79,12 @@ extern zend_module_entry yaml_module_entry;
 
 ZEND_BEGIN_MODULE_GLOBALS(yaml)
 	zend_bool decode_binary;
-	long decode_timestamp;
+	zend_long decode_timestamp;
 	zend_bool decode_php;
 	zval *timestamp_decoder;
 	zend_bool output_canonical;
-	long output_indent;
-	long output_width;
-#ifdef IS_UNICODE
-	UConverter *orig_runtime_encoding_conv;
-#endif
+	zend_long output_indent;
+	zend_long output_width;
 ZEND_END_MODULE_GLOBALS(yaml)
 
 ZEND_EXTERN_MODULE_GLOBALS(yaml)
