@@ -33,22 +33,11 @@
  */
 
 #include "php_yaml.h"
-#include "zval_refcount.h"		/* for PHP < 5.3 */
 #include "php_yaml_int.h"
 
-/* {{{ local macros
- */
-#if ZEND_EXTENSION_API_NO < 220060519
-#define PHP_GINIT_FUNCTION(yaml) \
-  void php_yaml_init_globals(zend_yaml_globals *yaml_globals)
-#endif
-
-/* }}} */
-
-
 /* {{{ local prototypes
  */
-static int php_yaml_check_callbacks(HashTable *callbacks TSRMLS_DC);
+static int php_yaml_check_callbacks(HashTable *callbacks);
 
 static PHP_MINIT_FUNCTION(yaml);
 static PHP_MSHUTDOWN_FUNCTION(yaml);
@@ -72,16 +61,12 @@ ZEND_DECLARE_MODULE_GLOBALS(yaml);
 
 /* {{{ ini entries */
 
-#ifndef ZEND_ENGINE_2
-#define OnUpdateLong OnUpdateInt
-#endif
-
 PHP_INI_BEGIN()
 	STD_PHP_INI_ENTRY("yaml.decode_binary", "0", PHP_INI_ALL, OnUpdateBool,
 			decode_binary, zend_yaml_globals, yaml_globals)
 	STD_PHP_INI_ENTRY("yaml.decode_timestamp", "0", PHP_INI_ALL, OnUpdateLong,
 			decode_timestamp, zend_yaml_globals, yaml_globals)
-	STD_PHP_INI_ENTRY("yaml.decode_php", "1", PHP_INI_ALL, OnUpdateBool,
+	STD_PHP_INI_ENTRY("yaml.decode_php", "0", PHP_INI_ALL, OnUpdateBool,
 			decode_php, zend_yaml_globals, yaml_globals)
 	STD_PHP_INI_ENTRY("yaml.output_canonical", "0", PHP_INI_ALL, OnUpdateBool,
 			output_canonical, zend_yaml_globals, yaml_globals)
@@ -154,23 +139,16 @@ static zend_function_entry yaml_functions[] = {
 
 
 /* {{{ cross-extension dependencies */
-
-#if ZEND_EXTENSION_API_NO >= 220050617
 static zend_module_dep yaml_deps[] = {
 	ZEND_MOD_OPTIONAL("date") {NULL, NULL, NULL, 0}
 };
-#endif
 /* }}} */
 
 
 zend_module_entry yaml_module_entry = {	/* {{{ */
-#if ZEND_EXTENSION_API_NO >= 220050617
 	STANDARD_MODULE_HEADER_EX,
 	NULL,
 	yaml_deps,
-#else
-	STANDARD_MODULE_HEADER,
-#endif
 	"yaml",
 	yaml_functions,
 	PHP_MINIT(yaml),
@@ -179,15 +157,11 @@ zend_module_entry yaml_module_entry = {	/* {{{ */
 	NULL, /* RSHUTDOWN */
 	PHP_MINFO(yaml),
 	PHP_YAML_VERSION,
-#if ZEND_EXTENSION_API_NO >= 220060519
 	PHP_MODULE_GLOBALS(yaml),
 	PHP_GINIT(yaml),
 	NULL,
 	NULL,
 	STANDARD_MODULE_PROPERTIES_EX
-#else
-	STANDARD_MODULE_PROPERTIES
-#endif
 };
 /* }}} */
 
@@ -200,9 +174,6 @@ ZEND_GET_MODULE(yaml)
 /* {{{ PHP_MINIT_FUNCTION */
 static PHP_MINIT_FUNCTION(yaml)
 {
-#if ZEND_EXTENSION_API_NO < 220060519
-	ZEND_INIT_MODULE_GLOBALS(yaml, php_yaml_init_globals, NULL)
-#endif
 	REGISTER_INI_ENTRIES();
 
 	/* node style constants */
@@ -296,14 +267,11 @@ static PHP_GINIT_FUNCTION(yaml)
 {
 	yaml_globals->decode_binary = 0;
 	yaml_globals->decode_timestamp = 0;
-	yaml_globals->decode_php = 1;
+	yaml_globals->decode_php = 0;
 	yaml_globals->timestamp_decoder = NULL;
 	yaml_globals->output_canonical = 0;
 	yaml_globals->output_indent = 2;
 	yaml_globals->output_width = 80;
-#ifdef IS_UNICODE
-	yaml_globals->orig_runtime_encoding_conv = NULL;
-#endif
 }
 /* }}} */
 
@@ -311,94 +279,40 @@ static PHP_GINIT_FUNCTION(yaml)
 /* {{{ php_yaml_check_callbacks()
  * Validate user supplied callback array contents
  */
-static int php_yaml_check_callbacks(HashTable *callbacks TSRMLS_DC)
+static int php_yaml_check_callbacks(HashTable *callbacks)
 {
-	zval **entry = { 0 };
-#ifdef IS_UNICODE
-	zstr key;
-#else
-	char *key = { 0 };
-#endif
-	uint key_len = 0;
-	ulong idx = 0L;
+	zval *entry;
+	zend_string *key;
 
-	zend_hash_internal_pointer_reset(callbacks);
+	ZEND_HASH_FOREACH_STR_KEY_VAL(callbacks, key, entry) {
+		if (key) {
+			zend_string *name;
 
-	while (SUCCESS ==
-			zend_hash_get_current_data(callbacks, (void **) &entry)) {
-		int key_type = zend_hash_get_current_key_ex(
-				callbacks, &key, &key_len, &idx, 0, NULL);
-
-#ifdef IS_UNICODE
-		if (key_type == HASH_KEY_IS_STRING || key_type == HASH_KEY_IS_UNICODE) {
-			zval name;
-			int type =
-				(key_type == HASH_KEY_IS_STRING) ? IS_STRING : IS_UNICODE;
-
-			INIT_ZVAL(name);
-
-			if (!zend_is_callable(*entry, 0, &name)) {
-				if (Z_TYPE(name) == IS_UNICODE ||
-					Z_TYPE(name) == IS_STRING) {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING,
-							"Callback for tag '%R', '%R' is not valid",
-							type, key, Z_TYPE(name), Z_UNIVAL(name));
-
-				} else {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING,
-							"Callback for tag '%R' is not valid",
-							type, key);
-				}
-
-				zval_dtor(&name);
-				return FAILURE;
-			}
-
-			if (ZEND_U_EQUAL(type, key, key_len - 1,
-						YAML_TIMESTAMP_TAG, sizeof(YAML_TIMESTAMP_TAG) - 1)) {
-				YAML_G(timestamp_decoder) = *entry;
-			}
-
-			zval_dtor(&name);
-
-		} else {
-			php_error_docref(NULL TSRMLS_CC, E_NOTICE,
-					"Callback key should be a string");
-		}
-#else
-		if (key_type == HASH_KEY_IS_STRING) {
-			char *name;
-
-			if (!ZEND_IS_CALLABLE(*entry, 0, &name)) {
+			if (!zend_is_callable(entry, 0, &name)) {
 				if (name != NULL) {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING,
+					php_error_docref(NULL, E_WARNING,
 							"Callback for tag '%s', '%s' is not valid",
-							key, name);
-					efree(name);
+							key->val, name->val);
+					zend_string_release(name);
 
 				} else {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING,
-							"Callback for tag '%s' is not valid", key);
+					php_error_docref(NULL, E_WARNING,
+							"Callback for tag '%s' is not valid", key->val);
 				}
 				return FAILURE;
+			} else {
+				zend_string_release(name);
 			}
 
-			if (!strcmp(key, YAML_TIMESTAMP_TAG)) {
-				YAML_G(timestamp_decoder) = *entry;
-			}
-
-			if (name != NULL) {
-				efree(name);
+			if (!memcmp(key->val, YAML_TIMESTAMP_TAG, sizeof(YAML_TIMESTAMP_TAG))) {
+				YAML_G(timestamp_decoder) = entry;
 			}
 
 		} else {
-			php_error_docref(NULL TSRMLS_CC, E_NOTICE,
+			php_error_docref(NULL, E_NOTICE,
 					"Callback key should be a string");
 		}
-#endif
-
-		zend_hash_move_forward(callbacks);
-	}
+	} ZEND_HASH_FOREACH_END();
 
 	return SUCCESS;
 }
@@ -409,43 +323,30 @@ static int php_yaml_check_callbacks(HashTable *callbacks TSRMLS_DC)
  Takes a YAML encoded string and converts it to a PHP variable. */
 PHP_FUNCTION(yaml_parse)
 {
-	char *input = { 0 };
-	int input_len = 0;
-	long pos = 0;
+	zend_string *input;
+	zend_long pos = 0;
 	zval *zndocs = { 0 };
 	zval *zcallbacks = { 0 };
 
 	parser_state_t state;
-	zval *yaml = { 0 };
-	long ndocs = 0;
+	zval yaml;
+	zend_long ndocs = 0;
 
 	memset(&state, 0, sizeof(state));
 	state.have_event = 0;
-	state.aliases = NULL;
 	state.callbacks = NULL;
 
-#ifdef IS_UNICODE
-	YAML_G(orig_runtime_encoding_conv) = UG(runtime_encoding_conv);
-#endif
 	YAML_G(timestamp_decoder) = NULL;
 
-#ifdef IS_UNICODE
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
-					"s&|lza/", &input, &input_len, UG(utf8_conv), &pos,
-					&zndocs, &zcallbacks)) {
-		return;
-	}
-#else
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
-					"s|lza/", &input, &input_len, &pos, &zndocs,
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(),
+					"S|lz/a/", &input, &pos, &zndocs,
 					&zcallbacks)) {
 		return;
 	}
-#endif
 
 	if (zcallbacks != NULL) {
 		state.callbacks = Z_ARRVAL_P(zcallbacks);
-		if (FAILURE == php_yaml_check_callbacks(state.callbacks TSRMLS_CC)) {
+		if (FAILURE == php_yaml_check_callbacks(state.callbacks)) {
 			RETURN_FALSE;
 		}
 
@@ -455,39 +356,29 @@ PHP_FUNCTION(yaml_parse)
 		state.eval_func = eval_scalar;
 	}
 
-#ifdef IS_UNICODE
-	UG(runtime_encoding_conv) = UG(utf8_conv);
-#endif
-
 	yaml_parser_initialize(&state.parser);
-	yaml_parser_set_input_string(
-			&state.parser, (unsigned char *) input, (size_t) input_len);
+	yaml_parser_set_input_string(&state.parser, (unsigned char *)input->val, input->len);
 
 
 	if (pos < 0) {
-		yaml = php_yaml_read_all(&state, &ndocs TSRMLS_CC);
-
+		php_yaml_read_all(&state, &ndocs, &yaml);
 	} else {
-		yaml = php_yaml_read_partial(&state, pos, &ndocs TSRMLS_CC);
+		php_yaml_read_partial(&state, pos, &ndocs, &yaml);
 	}
 
 	yaml_parser_delete(&state.parser);
 
-#ifdef IS_UNICODE
-	UG(runtime_encoding_conv) = YAML_G(orig_runtime_encoding_conv);
-#endif
-
 	if (zndocs != NULL) {
 		/* copy document count to var user sent in */
-		zval_dtor(zndocs);
+		zval_ptr_dtor(zndocs);
 		ZVAL_LONG(zndocs, ndocs);
 	}
 
-	if (NULL == yaml) {
+	if (Z_TYPE_P(&yaml) == IS_UNDEF) {
 		RETURN_FALSE;
 	}
 
-	RETURN_ZVAL(yaml, 1, 1);
+	RETURN_ZVAL(&yaml, 1, 1);
 }
 /* }}} yaml_parse */
 
@@ -497,8 +388,8 @@ PHP_FUNCTION(yaml_parse)
 PHP_FUNCTION(yaml_parse_file)
 {
 	char *filename = { 0 };
-	int filename_len = 0;
-	long pos = 0;
+	size_t filename_len = 0;
+	zend_long pos = 0;
 	zval *zndocs = { 0 };
 	zval *zcallbacks = { 0 };
 
@@ -506,37 +397,24 @@ PHP_FUNCTION(yaml_parse_file)
 	FILE *fp = { 0 };
 
 	parser_state_t state;
-	zval *yaml = { 0 };
-	long ndocs = 0;
+	zval yaml;
+	zend_long ndocs = 0;
 
 	memset(&state, 0, sizeof(state));
 	state.have_event = 0;
-	state.aliases = NULL;
 	state.callbacks = NULL;
 
-#ifdef IS_UNICODE
-	YAML_G(orig_runtime_encoding_conv) = UG(runtime_encoding_conv);
-#endif
 	YAML_G(timestamp_decoder) = NULL;
 
-#ifdef IS_UNICODE
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
-					"s&|lza/", &filename, &filename_len,
-					ZEND_U_CONVERTER(UG(filesystem_encoding_conv)), &pos,
-					&zndocs, &zcallbacks)) {
-		return;
-	}
-#else
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(),
 					"s|lza/", &filename, &filename_len, &pos, &zndocs,
 					&zcallbacks)) {
 		return;
 	}
-#endif
 
 	if (zcallbacks != NULL) {
 		state.callbacks = Z_ARRVAL_P(zcallbacks);
-		if (FAILURE == php_yaml_check_callbacks(state.callbacks TSRMLS_CC)) {
+		if (FAILURE == php_yaml_check_callbacks(state.callbacks)) {
 			RETURN_FALSE;
 		}
 
@@ -547,7 +425,7 @@ PHP_FUNCTION(yaml_parse_file)
 	}
 
 	if (NULL == (stream = php_stream_open_wrapper(filename, "rb",
-			IGNORE_URL | ENFORCE_SAFE_MODE | REPORT_ERRORS | STREAM_WILL_CAST,
+			IGNORE_URL | REPORT_ERRORS | STREAM_WILL_CAST,
 			NULL))) {
 		RETURN_FALSE;
 	}
@@ -558,38 +436,30 @@ PHP_FUNCTION(yaml_parse_file)
 		RETURN_FALSE;
 	}
 
-#ifdef IS_UNICODE
-	UG(runtime_encoding_conv) = UG(utf8_conv);
-#endif
-
 	yaml_parser_initialize(&state.parser);
 	yaml_parser_set_input_file(&state.parser, fp);
 
 	if (pos < 0) {
-		yaml = php_yaml_read_all(&state, &ndocs TSRMLS_CC);
+		php_yaml_read_all(&state, &ndocs, &yaml);
 
 	} else {
-		yaml = php_yaml_read_partial(&state, pos, &ndocs TSRMLS_CC);
+		php_yaml_read_partial(&state, pos, &ndocs, &yaml);
 	}
 
 	yaml_parser_delete(&state.parser);
 	php_stream_close(stream);
 
-#ifdef IS_UNICODE
-	UG(runtime_encoding_conv) = YAML_G(orig_runtime_encoding_conv);
-#endif
-
 	if (zndocs != NULL) {
 		/* copy document count to var user sent in */
 		zval_dtor(zndocs);
 		ZVAL_LONG(zndocs, ndocs);
 	}
 
-	if (NULL == yaml) {
+	if (Z_TYPE_P(&yaml) == IS_UNDEF) {
 		RETURN_FALSE;
 	}
 
-	RETURN_ZVAL(yaml, 1, 1);
+	RETURN_ZVAL(&yaml, 1, 1);
 }
 /* }}} yaml_parse_file */
 
@@ -599,37 +469,33 @@ PHP_FUNCTION(yaml_parse_file)
 PHP_FUNCTION(yaml_parse_url)
 {
 	char *url = { 0 };
-	int url_len = 0;
-	long pos = 0;
+	size_t url_len = 0;
+	zend_long pos = 0;
 	zval *zndocs = { 0 };
 	zval *zcallbacks = { 0 };
 
 	php_stream *stream = { 0 };
-	char *input = { 0 };
+	zend_string *input;
 	size_t size = 0;
 
 	parser_state_t state;
-	zval *yaml = { 0 };
-	long ndocs = 0;
+	zval yaml;
+	zend_long ndocs = 0;
 
 	memset(&state, 0, sizeof(state));
 	state.have_event = 0;
-	state.aliases = NULL;
 	state.callbacks = NULL;
 
-#ifdef IS_UNICODE
-	YAML_G(orig_runtime_encoding_conv) = UG(runtime_encoding_conv);
-#endif
 	YAML_G(timestamp_decoder) = NULL;
 
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC,
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(),
 				"s|lza/", &url, &url_len, &pos, &zndocs, &zcallbacks)) {
 		return;
 	}
 
 	if (zcallbacks != NULL) {
 		state.callbacks = Z_ARRVAL_P(zcallbacks);
-		if (FAILURE == php_yaml_check_callbacks(state.callbacks TSRMLS_CC)) {
+		if (FAILURE == php_yaml_check_callbacks(state.callbacks)) {
 			RETURN_FALSE;
 		}
 
@@ -639,39 +505,27 @@ PHP_FUNCTION(yaml_parse_url)
 		state.eval_func = eval_scalar;
 	}
 
-	if (NULL == (stream = php_stream_open_wrapper(url, "rb",
-					ENFORCE_SAFE_MODE | REPORT_ERRORS, NULL))) {
+	if (NULL == (stream = php_stream_open_wrapper(url, "rb", REPORT_ERRORS, NULL))) {
 		RETURN_FALSE;
 	}
 
-#ifdef IS_UNICODE
-	size = php_stream_copy_to_mem(
-			stream, (void **) &input, PHP_STREAM_COPY_ALL, 0);
-#else
-	size = php_stream_copy_to_mem(stream, &input, PHP_STREAM_COPY_ALL, 0);
-#endif
-
-#ifdef IS_UNICODE
-	UG(runtime_encoding_conv) = UG(utf8_conv);
-#endif
+	input = php_stream_copy_to_mem(stream, PHP_STREAM_COPY_ALL, 0);
+	php_stream_close(stream);
+	if (input == NULL) {
+		RETURN_FALSE;
+	}
 
 	yaml_parser_initialize(&state.parser);
 	yaml_parser_set_input_string(&state.parser, (unsigned char *)input, size);
 
 	if (pos < 0) {
-		yaml = php_yaml_read_all(&state, &ndocs TSRMLS_CC);
-
+		php_yaml_read_all(&state, &ndocs, &yaml);
 	} else {
-		yaml = php_yaml_read_partial(&state, pos, &ndocs TSRMLS_CC);
+		php_yaml_read_partial(&state, pos, &ndocs, &yaml);
 	}
 
 	yaml_parser_delete(&state.parser);
-	php_stream_close(stream);
-	efree(input);
-
-#ifdef IS_UNICODE
-	UG(runtime_encoding_conv) = YAML_G(orig_runtime_encoding_conv);
-#endif
+	zend_string_release(input);
 
 	if (zndocs != NULL) {
 		/* copy document count to var user sent in */
@@ -679,11 +533,11 @@ PHP_FUNCTION(yaml_parse_url)
 		ZVAL_LONG(zndocs, ndocs);
 	}
 
-	if (NULL == yaml) {
+	if (Z_TYPE_P(&yaml) == IS_UNDEF) {
 		RETURN_FALSE;
 	}
 
-	RETURN_ZVAL(yaml, 1, 1);
+	RETURN_ZVAL(&yaml, 1, 1);
 }
 /* }}} yaml_parse_url */
 
@@ -693,22 +547,22 @@ PHP_FUNCTION(yaml_parse_url)
 PHP_FUNCTION(yaml_emit)
 {
 	zval *data = { 0 };
-	long encoding = YAML_ANY_ENCODING;
-	long linebreak = YAML_ANY_BREAK;
+	zend_long encoding = YAML_ANY_ENCODING;
+	zend_long linebreak = YAML_ANY_BREAK;
 	zval *zcallbacks = { 0 };
 	HashTable *callbacks = { 0 };
 
 	yaml_emitter_t emitter = { 0 };
-	smart_str str = { 0 };
+	smart_string str = { 0 };
 
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS()TSRMLS_CC, "z/|lla/",
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "z/|lla/",
 				&data, &encoding, &linebreak, &zcallbacks)) {
 		return;
 	}
 
 	if (zcallbacks != NULL) {
 		callbacks = Z_ARRVAL_P(zcallbacks);
-		if (FAILURE == php_yaml_check_callbacks(callbacks TSRMLS_CC)) {
+		if (FAILURE == php_yaml_check_callbacks(callbacks)) {
 			RETURN_FALSE;
 		}
 	}
@@ -725,60 +579,48 @@ PHP_FUNCTION(yaml_emit)
 
 	if (SUCCESS == php_yaml_write_impl(
 				&emitter, data, (yaml_encoding_t) encoding,
-				callbacks TSRMLS_CC)) {
-#ifdef IS_UNICODE
-		RETVAL_U_STRINGL(UG(utf8_conv), str.c, str.len, ZSTR_DUPLICATE);
-#else
-		RETVAL_STRINGL(str.c, str.len, 1);
-#endif
+				callbacks)) {
+		RETVAL_STRINGL(str.c, str.len);
 
 	} else {
 		RETVAL_FALSE;
 	}
 
 	yaml_emitter_delete(&emitter);
-	smart_str_free(&str);
+	smart_string_free(&str);
 }
 /* }}} yaml_emit */
 
 
-/* {{{ proto bool yaml_emit_file(string filename, mixed data[, string encoding[, string linebreak[, array callbacks]]])
+/* {{{ proto bool yaml_emit_file(string filename, mixed data[, int encoding[, int linebreak[, array callbacks]]])
    */
 PHP_FUNCTION(yaml_emit_file)
 {
-	char *filename = { 0 };
-	int filename_len = 0;
+	zend_string *filename = { 0 };
 	php_stream *stream = { 0 };
 	FILE *fp = { 0 };
 	zval *data = { 0 };
-	const char *encoding = { 0 };
-	int encoding_len = 0;
-	const char *linebreak = { 0 };
-	int linebreak_len = 0;
+	zend_long encoding = YAML_ANY_ENCODING;
+	zend_long linebreak = YAML_ANY_BREAK;
 	zval *zcallbacks = { 0 };
 	HashTable *callbacks = { 0 };
 
 	yaml_emitter_t emitter = { 0 };
 
-#ifdef IS_UNICODE
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s&z/|ssa/",
-			&filename, &filename_len,
-			ZEND_U_CONVERTER(UG(filesystem_encoding_conv)), &data,
-			&encoding, &encoding_len, &linebreak, &zcallbacks,
-			&linebreak_len)) {
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "Sz/|lla/",
+			&filename, &data, &encoding, &linebreak, &zcallbacks)) {
 		return;
 	}
-#else
-	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sz/|ssa/",
-			&filename, &filename_len, &data, &encoding,
-			&encoding_len, &linebreak, &zcallbacks,
-			&linebreak_len)) {
-		return;
-	}
-#endif
 
-	if (NULL == (stream = php_stream_open_wrapper(filename, "wb",
-			IGNORE_URL | ENFORCE_SAFE_MODE | REPORT_ERRORS | STREAM_WILL_CAST,
+	if (zcallbacks != NULL) {
+		callbacks = Z_ARRVAL_P(zcallbacks);
+		if (FAILURE == php_yaml_check_callbacks(callbacks)) {
+			RETURN_FALSE;
+		}
+	}
+
+	if (NULL == (stream = php_stream_open_wrapper(filename->val, "wb",
+			IGNORE_URL | REPORT_ERRORS | STREAM_WILL_CAST,
 			NULL))) {
 		RETURN_FALSE;
 	}
@@ -799,7 +641,7 @@ PHP_FUNCTION(yaml_emit_file)
 	yaml_emitter_set_unicode(&emitter, YAML_ANY_ENCODING != encoding);
 
 	RETVAL_BOOL((SUCCESS == php_yaml_write_impl(
-			&emitter, data, YAML_ANY_ENCODING, callbacks TSRMLS_CC)));
+			&emitter, data, YAML_ANY_ENCODING, callbacks)));
 
 	yaml_emitter_delete(&emitter);
 	php_stream_close(stream);
